package com.ycwl.basic.service.impl.pc;

import cn.hutool.core.bean.BeanUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.constant.NumberConstant;
import com.ycwl.basic.enums.OrderStateEnum;
import com.ycwl.basic.enums.SourceTypeNameEnum;
import com.ycwl.basic.mapper.pc.OrderMapper;
import com.ycwl.basic.mapper.pc.SourceMapper;
import com.ycwl.basic.mapper.pc.VideoMapper;
import com.ycwl.basic.model.mobile.goods.GoodsPriceQueryReq;
import com.ycwl.basic.model.pc.member.resp.MemberRespVO;
import com.ycwl.basic.model.pc.order.entity.OrderItemEntity;
import com.ycwl.basic.model.pc.order.req.OrderAddOrUpdateReq;
import com.ycwl.basic.model.pc.order.req.OrderItemDTO;
import com.ycwl.basic.model.pc.order.req.OrderReqQuery;
import com.ycwl.basic.model.pc.order.resp.OrderAppRespVO;
import com.ycwl.basic.model.pc.order.resp.OrderItemVO;
import com.ycwl.basic.model.pc.order.resp.OrderRespVO;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.model.wx.WXPayOrderReqVO;
import com.ycwl.basic.model.wx.WxPayRespVO;
import com.ycwl.basic.service.mobile.AppMemberService;
import com.ycwl.basic.service.mobile.GoodsService;
import com.ycwl.basic.service.mobile.WxPayService;
import com.ycwl.basic.service.pc.OrderService;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.BigDecimalUtil;
import com.ycwl.basic.utils.SnowFlakeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @Author：longbinbin
 * @Date：2024/12/3 13:54
 */
@Slf4j
@Service
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private VideoMapper videoMapper;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private WxPayService wxPayService;

    @Autowired
    private AppMemberService memberService;
    @Autowired
    private GoodsService goodsService;

    @Override
    public ApiResponse<PageInfo<OrderRespVO>> pageQuery(OrderReqQuery query) {
        PageHelper.startPage(query.getPageNum(), query.getPageSize());
        List<OrderRespVO> list = orderMapper.list(query);
        PageInfo<OrderRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<OrderRespVO>> list(OrderReqQuery query) {
        return ApiResponse.success(orderMapper.list(query));
    }

    @Override
    public ApiResponse<OrderRespVO> detail(Long orderId) {
        return ApiResponse.success(orderMapper.getById(orderId));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ApiResponse<WxPayRespVO> add(OrderAddOrUpdateReq order) throws Exception {
        Long orderId = SnowFlakeUtil.getLongId();
        order.setId(orderId);
        int add = orderMapper.add(order);
        if (add == NumberConstant.ZERO) {
            return ApiResponse.fail("订单添加失败");
        } else {
            List<OrderItemDTO> orderItemList = order.getOrderItemList();
            List<OrderItemEntity> orderItems = new ArrayList<>();
            orderItemList.forEach(orderItemDTO -> {
                OrderItemEntity orderItemEntity = new OrderItemEntity();
                BeanUtil.copyProperties(orderItemDTO, orderItemEntity);
                orderItemEntity.setOrderId(orderId);
            });
            int addOrderItems = orderMapper.addOrderItems(orderItems);
            if (addOrderItems == NumberConstant.ZERO) {
                log.error("订单明细添加失败");
                return ApiResponse.fail("订单添加失败");
            }

            /* 成片的时候子订单只会有一个，素材的时候 子订单多个且价格与子订单数量无关，只与子订单商品对应景区定义的价格相同*/
            GoodsPriceQueryReq goodsPriceQueryReq = new GoodsPriceQueryReq();
            Integer goodsType = order.getOrderItemList().get(NumberConstant.ZERO).getGoodsType();
            Long goodsId = order.getOrderItemList().get(NumberConstant.ZERO).getGoodsId();

            String goodsName = null;
            if (goodsType == NumberConstant.ONE) {
                VideoRespVO videoRespVO = videoMapper.getById(goodsId);
                goodsName = videoRespVO.getTemplateName();
            } else if (goodsType == NumberConstant.TWO) {
                SourceRespVO sourceRespVO = sourceMapper.getById(goodsId);
                goodsName= SourceTypeNameEnum.getRemark( sourceRespVO.getType());
            }
            BigDecimal price = goodsService.queryPrice(goodsPriceQueryReq.setGoodsId(goodsId)
                    .setGoodsType(goodsType)).getData();

            MemberRespVO memberRespVO = memberService.getUserInfo().getData();
            WXPayOrderReqVO wxPayOrderReqVO = new WXPayOrderReqVO();
            wxPayOrderReqVO.setOpenId(memberRespVO.getOpenId())
                    .setOrderSn(orderId)
                    .setTotalPrice(BigDecimalUtil.convertToCents(price))
                    .setGoodsName(goodsName);
            return   ApiResponse.success(wxPayService.createOrder(wxPayOrderReqVO));
        }
    }

    @Override
    public ApiResponse<Integer> update(OrderAddOrUpdateReq query) {
        int update = orderMapper.update(query);
        if (update == NumberConstant.ZERO) {
            return ApiResponse.fail("订单更新失败");
        }
        return ApiResponse.success(update);
    }

    /**
     * 更新订单支付状态
     *
     * @param orderSn 订单编号也就是订单的ID
     */
    @Override
    public void updateOrderState(Long orderSn, OrderStateEnum orderStateEnum, String refundReason) {
        OrderAddOrUpdateReq orderAddOrUpdateReq = new OrderAddOrUpdateReq();
        orderAddOrUpdateReq.setId(orderSn);
        if (orderStateEnum.getType() == NumberConstant.ONE) {
            orderAddOrUpdateReq.setRefundStatus(orderStateEnum.getState());
            orderAddOrUpdateReq.setRefundAt(new Date());
            if (Objects.nonNull(refundReason)) {
                orderAddOrUpdateReq.setRefundReason(refundReason);
            }
        } else if (orderStateEnum.getType() == NumberConstant.TWO) {
            int state = orderStateEnum.getState();
            orderAddOrUpdateReq.setPayAt(new Date());
            orderAddOrUpdateReq.setStatus(orderStateEnum.getState());
            if (state == OrderStateEnum.CANCELED.getState()) {
                orderAddOrUpdateReq.setCancelAt(new Date());
            }
        }
        orderMapper.update(orderAddOrUpdateReq);
    }

    @Override
    public ApiResponse<?> getOrderCountByUserId(Long userId) {
        OrderReqQuery query = new OrderReqQuery();
        query.setMemberId(userId);
        return orderMapper.getOrderCount(query);
    }

    @Override
    public ApiResponse<PageInfo<OrderAppRespVO>> appPageQuery(OrderReqQuery orderReqQuery) {
        PageHelper.startPage(orderReqQuery.getPageNum(), orderReqQuery.getPageSize());
        List<OrderAppRespVO> list = orderMapper.appList(orderReqQuery);
        for (OrderAppRespVO appRespVO : list) {
            List<OrderItemVO> orderItemList = appRespVO.getOrderItemList();
            if (orderItemList != null && !orderItemList.isEmpty()) {
                OrderItemVO itemVO = orderItemList.get(NumberConstant.ZERO);
                appRespVO.setScenicName(itemVO.getScenicName());
                appRespVO.setGoodsName(itemVO.getGoodsName());
            }
        }
        PageInfo<OrderAppRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<OrderAppRespVO> appDetail(Long id) {
        OrderAppRespVO orderAppRespVO = orderMapper.appDetail(id);
        List<OrderItemVO> orderItemList = orderAppRespVO.getOrderItemList();
        if (orderItemList != null && !orderItemList.isEmpty()) {
            OrderItemVO itemVO = orderItemList.get(NumberConstant.ZERO);
            orderAppRespVO.setScenicName(itemVO.getScenicName());
            orderAppRespVO.setGoodsName(itemVO.getGoodsName());
        }
        return ApiResponse.success(orderAppRespVO);
    }

}
